# contains class for flashcards with question and answers
from pygame.sprite import Sprite
from pygame.font import SysFont
from pygame.image import load


class Card(Sprite):
    def __init__(self):
        super().__init__()
        self.x, self.y = 110, 120
        self.font, self.text_sprite, self.text_rect = None, None, None
        self.question, self.answer = None, None
        self.rect, self.image = None, None
        self.front, self.back = None, None
        self.flipped = False

    def set_fields(self, question: str, answer: str):
        self.question, self.answer = question, answer
        return self

    def set_pos(self, x: int, y: int):
        self.x, self.y = x, y

    def set_imgs(self, front: str, back: str):
        self.front, self.back = front, back
        self.refresh_img()

    def load_text(self, font):
        self.font = SysFont(font, 30, italic=False, bold=False)
        self.text_sprite = self.font.render(self.question, True, (255, 255, 255))
        self.text_rect = self.text_sprite.get_rect()

    def render_text(self):
        if self.flipped:
            txt = self.answer
        else:
            txt = self.question
        self.text_rect.center = (self.x + 250, self.y + 100)
        self.text_sprite = self.font.render(txt, True, (255, 255, 255))
        return self.text_sprite, self.text_rect

    def refresh_img(self):
        if self.flipped:
            self.image = load(self.back)
        else:
            self.image = load(self.front)
        self.rect = self.image.get_rect()

    def flip(self):
        self.flipped = not self.flipped
        self.refresh_img()

    def update(self) -> None:
        """Override pygame's default sprite update()"""
        self.rect.x, self.rect.y = self.x, self.y




from pygame.font import SysFont


class Text:
    """Manages all text to display onscreen (except for flashcards text)"""
    def __init__(self, text, font, size, red, green, blue):
        self.text = text
        self.field = SysFont(font, size, italic=False, bold=False)
        self.sprite = self.field.render(self.text, True, (red, green, blue))
        self.rect = self.sprite.get_rect()



from .card import Card
from pygame.sprite import Group as SpriteGroup


class Deck:
    def __init__(self):
        self.cards = SpriteGroup()
        self.card_in_focus = SpriteGroup()

    def add_card(self, question: str, answer: str):
        """:param question the question for the Card to add to the deck
            :param answer the Card's answer"""
        self.cards.add(Card().set_fields(question, answer))
        return self

    def add_card_from_list(self, add: [str]):
        """:param add list of two strings formatted [question, answer]"""
        self.cards.add(Card().set_fields(add[0], add[1]))
        return self

    def add_card_from_obj(self, add: Card):
        """:param add the Card object to add to this deck"""
        self.cards.add(add)
        return self

    def init_cards(self, front: str, back: str, font: str):
        """Set the background images for all self.cards
        :param front .jpg (500x250) pixels to use for the front
        :param back .jpg to use for the back
        :param font name of the pygame font to load"""
        for card in self.cards:
            card.set_imgs(front, back)
            card.load_text(font)

    def get_first_card(self) -> SpriteGroup:
        temp_group = SpriteGroup()
        temp_group.add(self.cards.sprites()[0])
        return temp_group

    def move_to_back(self, mv_card: Card):
        self.unflip()
        self.cards.remove(mv_card)
        self.cards.add(mv_card)

    def move_first_to_back(self):
        return self.move_to_back(self.cards.sprites()[0])

    def move_to_front(self, mv_card: Card):
        self.unflip()
        grouplist = []
        for c in self.cards:
            grouplist.append(c)
        grouplist.remove(mv_card)
        grouplist.insert(0, mv_card)
        tempgroup = SpriteGroup()
        for c in grouplist:
            tempgroup.add(c)
        self.cards = tempgroup

    def unflip(self):
        for card in self.cards:
            if card.flipped:
                card.flip()

    def move_last_to_front(self):
        return self.move_to_front(self.cards.sprites()[-1])

    def print(self):
        """Print each question: answer in the deck"""
        for card in self.cards:
            print(f"{card.question}: {card.answer}")



import pygame
from ..flashcards import Deck, Card
from pygame.sprite import Group as SpriteGroup


class GUI:
    def __init__(self, opts):
        self.screen, self.clock = None, None
        self.window_name = opts.title
        self.dimensions = (opts.w, opts.h)
        self.FPS = opts.fps

    def reset_clock(self):
        self.clock = pygame.time.Clock()

    def init_screen(self):
        pygame.init()
        self.reset_clock()
        self.screen = pygame.display.set_mode(self.dimensions)
        pygame.display.set_caption(self.window_name)
        return self

    def handle_events(self, deck: Deck):
        for event in pygame.event.get():
            keys = pygame.key.get_pressed()
            if event.type == pygame.MOUSEBUTTONDOWN:
                deck.cards.sprites()[0].flip()
            if keys[pygame.K_RIGHT]:
                deck.move_first_to_back()
            if keys[pygame.K_LEFT]:
                deck.move_last_to_front()
            if keys[pygame.K_ESCAPE] or event.type == pygame.QUIT:
                return False, deck
        return True, deck

    def render(self, group: SpriteGroup):
        self.clock.tick(self.FPS)
        for card in group:
            card.update()
            sprite, rect = card.render_text()
            self.screen.blit(sprite, rect)
        pygame.display.update()

    def render_card(self, card: Card):
        self.clock.tick(self.FPS)
        card.update()
        sprite, rect = card.render_text()
        self.screen.blit(sprite, rect)
        pygame.display.update()

    @staticmethod
    def get_mouse_pos() -> (int, int):
        return pygame.mouse.get_pos()

    @staticmethod
    def quit():
        pygame.quit()



from flashquiz.flashcards.parser import from_csv
from flashquiz.args import handle_args
from flashquiz.gui import GUI


def main():
    args = handle_args()
    game = GUI(args).init_screen()
    deck = from_csv(args.file)
    deck.init_cards(args.cards_front, args.cards_back, args.font)
    run = True
    while run:
        game.screen.fill((0, 0, 0))

        card_in_focus = deck.get_first_card()
        card_in_focus.draw(game.screen)
        game.render(card_in_focus)

        run, deck = game.handle_events(deck)

    game.quit()


if __name__ == '__main__':
    main()
